// decodable-nnet-online-for-hark.h
// If you want to put this file under the directory, it file is placed in under the "nnet" directory.
// Implements like "nnet2/online-nnet2-decodable.h".
// Implements like "nnet3/online-nnet3-decodable.h".

#ifndef KALDI_DECODABLE_NNET2_ONLINE_FOR_HARK_H_
#define KALDI_DECODABLE_NNET2_ONLINE_FOR_HARK_H_

#include "itf/online-feature-itf.h"
#include "itf/decodable-itf.h"
#include "hmm/transition-model.h"
// nnet2
#include "nnet2/am-nnet.h"
#include "nnet2/nnet-compute.h"

namespace kaldi {
namespace nnet2 {

struct DecodableNnet2OnlineForHarkOptions {
  kaldi::BaseFloat acoustic_scale;
  bool pad_input;
  kaldi::int32 max_nnet_batch_size;
  
  DecodableNnet2OnlineForHarkOptions():
      acoustic_scale(0.1),
      pad_input(true),
      max_nnet_batch_size(256) { }

  // for kaldi compatibility
  void Register(OptionsItf *opts) {
    opts->Register("acoustic-scale", &acoustic_scale,
                   "Scaling factor for acoustic likelihoods");
    opts->Register("pad-input", &pad_input,
                   "If true, pad acoustic features with required acoustic context "
                   "past edges of file.");
    opts->Register("max-nnet-batch-size", &max_nnet_batch_size,
                   "Maximum batch size we use in neural-network decodable object, "
                   "in cases where we are not constrained by currently available "
                   "frames (this will rarely make a difference)");
  }
  // for avoid of option name conflict in multi nnet type support
  void SetAcousticScale(kaldi::BaseFloat arg) {
    acoustic_scale = arg;
  }
  void SetPadInput(bool arg) {
    pad_input = arg;
  }
  void SetMaxNnetBatchSize(kaldi::int32 arg) {
    max_nnet_batch_size = arg;
  }
};

class DecodableNnet2OnlineForHark: public DecodableInterface {
 public:
  DecodableNnet2OnlineForHark(const kaldi::nnet2::AmNnet &nnet,
                              const kaldi::TransitionModel &trans_model,
                              const DecodableNnet2OnlineForHarkOptions &opts,
                              //kaldi::OnlineFeatureInterface *features_if);
                              std::shared_ptr<kaldi::OnlineFeatureInterface> features_if);

  /// Returns the scaled log likelihood
  virtual kaldi::BaseFloat LogLikelihood(kaldi::int32 frame, kaldi::int32 index);

  virtual bool IsLastFrame(kaldi::int32 frame) const;

  virtual kaldi::int32 NumFramesReady() const;

  /// Indices are one-based!  This is for compatibility with OpenFst.
  virtual kaldi::int32 NumIndices() const { return trans_model_.NumTransitionIds(); }

 private:
  void ComputeForFrame(kaldi::int32 frame);

  //kaldi::OnlineFeatureInterface *features_if_;
  std::shared_ptr<kaldi::OnlineFeatureInterface> features_if_;
  const kaldi::nnet2::AmNnet &nnet_;
  const kaldi::TransitionModel &trans_model_;
  DecodableNnet2OnlineForHarkOptions opts_;
  kaldi::CuVector<kaldi::BaseFloat> log_priors_;
  kaldi::int32 feat_dim_;
  kaldi::int32 left_context_;
  kaldi::int32 right_context_;
  kaldi::int32 num_pdfs_;
  kaldi::int32 begin_frame_;

  kaldi::Matrix<kaldi::BaseFloat> scaled_loglikes_;

  //KALDI_DISALLOW_COPY_AND_ASSIGN(DecodableNnet2OnlineForHark);
};


} // namespace nnet2
} // namespace kaldi

#endif // KALDI_DECODABLE_NNET2_ONLINE_FOR_HARK_H_
